------------------------------------------------------------
revno: 12569
revision-id: squid3@treenet.co.nz-20130602160118-deeqie8ln4uwt37e
parent: squid3@treenet.co.nz-20130602155000-9j9qykhek9xc6ke7
author: Alex Rousskov <rousskov@measurement-factory.com>
committer: Amos Jeffries <squid3@treenet.co.nz>
branch nick: 3.3
timestamp: Sun 2013-06-02 10:01:18 -0600
message:
  Ask for SSL key password when started with -N but without sslpassword_program.
  
  Do not give SSL a password-asking callback if sslpassword_program is not
  configured. Without a callback, OpenSSL itself asks for the password (which
  works if Squid runs in foreground because of -N).
  
  The fix applies to Ssl::readCertChainAndPrivateKeyFromFiles() context only.
  This is not the only place where we read private keys. Some other places are
  working correctly, but others may need more work. Also,
  Ssl::readCertChainAndPrivateKeyFromFiles() may not really work if
  sslpassword_program _is_ configured because "user data" pointer will be nil.
------------------------------------------------------------
# Bazaar merge directive format 2 (Bazaar 0.90)
# revision_id: squid3@treenet.co.nz-20130602160118-deeqie8ln4uwt37e
# target_branch: http://bzr.squid-cache.org/bzr/squid3/3.3
# testament_sha1: e656f44522edc3e98c12f3c63907a79f6d4ac028
# timestamp: 2013-06-02 16:08:15 +0000
# source_branch: http://bzr.squid-cache.org/bzr/squid3/3.3
# base_revision_id: squid3@treenet.co.nz-20130602155000-\
#   9j9qykhek9xc6ke7
# 
# Begin patch
=== modified file 'src/ssl/support.cc'
--- src/ssl/support.cc	2013-05-19 03:11:29 +0000
+++ src/ssl/support.cc	2013-06-02 16:01:18 +0000
@@ -1521,7 +1521,10 @@
         chain.reset(sk_X509_new_null());
     if (!chain)
         debugs(83, DBG_IMPORTANT, "WARNING: unable to allocate memory for cert chain");
-    pkey.reset(readSslPrivateKey(keyFilename, ssl_ask_password_cb));
+    // XXX: ssl_ask_password_cb needs SSL_CTX_set_default_passwd_cb_userdata()
+    // so this may not fully work iff Config.Program.ssl_password is set.
+    pem_password_cb *cb = ::Config.Program.ssl_password ? &ssl_ask_password_cb : NULL;
+    pkey.reset(readSslPrivateKey(keyFilename, cb));
     cert.reset(readSslX509CertificatesChain(certFilename, chain.get()));
     if (!pkey || !cert || !X509_check_private_key(cert.get(), pkey.get())) {
         pkey.reset(NULL);

